@@ -1,5 +1,15 @@
--- Copyright (c) David Amos, 2008-2009. All rights reserved.
+-- Copyright (c) David Amos, 2008-2012. All rights reserved.
 
+{-# LANGUAGE NoMonomorphismRestriction #-}
+
+-- |A module for doing arithmetic in permutation groups.
+--
+-- Group elements are represented as permutations of underlying sets, and are entered and displayed
+-- using a Haskell-friendly version of cycle notation. For example, the permutation (1 2 3)(4 5)
+-- would be entered as @p [[1,2,3],[4,5]]@, and displayed as [[1,2,3],[4,5]]. Permutations can be defined
+-- over arbitrary underlying sets (types), not just the integers.
+--
+-- If @g@ and @h@ are group elements, then the expressions @g*h@ and @g^-1@ calculate product and inverse respectively.
 module Math.Algebra.Group.PermutationGroup where
 
 import qualified Data.List as L
@@ -8,7 +18,10 @@ import qualified Data.Set as S
 
 import Math.Common.ListSet (toListSet, union, (\\) ) -- a version of union which assumes the arguments are ascending sets (no repeated elements)
 
-infix 8 ^-, ~^
+import Math.Core.Utils hiding (elts)
+import Math.Algebra.LinearAlgebra hiding (inverse) -- only needed for use in ghci
+
+infix 8 ~^
 
 rotateL (x:xs) = xs ++ [x]
 
@@ -18,6 +31,11 @@ rotateL (x:xs) = xs ++ [x]
 -- |A type for permutations, considered as functions or actions which can be performed on an underlying set.
 newtype Permutation a = P (M.Map a a) deriving (Eq,Ord)
 
+-- |Construct a permutation from a list of cycles.
+-- For example, @p [[1,2,3],[4,5]]@ returns the permutation that sends 1 to 2, 2 to 3, 3 to 1, 4 to 5, 5 to 4.
+p :: (Ord a) => [[a]] -> Permutation a
+p = fromCycles
+
 fromPairs xys | isValid   = fromPairs' xys
               | otherwise = error "Not a permutation"
     where (xs,ys) = unzip xys
@@ -37,27 +55,23 @@ supp (P g) = M.keys g
 -- (This is guaranteed not to contain fixed points provided the permutations have been constructed using the supplied constructors)
 
 -- |x .^ g returns the image of a vertex or point x under the action of the permutation g.
+-- For example, @1 .^ p [[1,2,3]]@ returns 2.
 -- The dot is meant to be a mnemonic for point or vertex.
-(.^) :: (Ord k) => k -> Permutation k -> k
+(.^) :: (Ord a) => a -> Permutation a -> a
 x .^ P g = case M.lookup x g of
            Just y  -> y
            Nothing -> x -- if x `notElem` supp (P g), then x is not moved
 
 -- |b -^ g returns the image of an edge or block b under the action of the permutation g.
+-- For example, @[1,2] -^ p [[1,4],[2,3]]@ returns [3,4].
 -- The dash is meant to be a mnemonic for edge or line or block.
-(-^) :: (Ord t) => [t] -> Permutation t -> [t]
+(-^) :: (Ord a) => [a] -> Permutation a -> [a]
 xs -^ g = L.sort [x .^ g | x <- xs]
 
 -- construct a permutation from cycles
 fromCycles cs = fromPairs $ concatMap fromCycle cs
     where fromCycle xs = zip xs (rotateL xs)
 
--- |Construct a permutation from a list of cycles.
--- For example, p [[1,2,3],[4,5]] returns the permutation that sends 1 to 2, 2 to 3, 3 to 1, 4 to 5, 5 to 4
-p :: (Ord a) => [[a]] -> Permutation a
-p cs = fromCycles cs
--- can't specify in pointfree style because of monomorphism restriction
-
 -- convert a permutation to cycles
 toCycles g = toCycles' $ supp g
     where toCycles' ys@(y:_) = let c = cycleOf g y in c : toCycles' (ys L.\\ c)
@@ -77,23 +91,25 @@ sign g = (-1)^(parity g)
 orderElt g = foldl lcm 1 $ map length $ toCycles g
 -- == order [g]
 
+-- |The Num instance is what enables us to write @g*h@ for the product of group elements and @1@ for the group identity.
+-- Unfortunately we can't of course give sensible definitions for the other functions declared in the Num typeclass.
 instance (Ord a, Show a) => Num (Permutation a) where
     g * h = fromPairs' [(x, x .^ g .^ h) | x <- supp g `union` supp h]
     -- signum = sign -- doesn't work, complains about no (+) instance
     fromInteger 1 = P $ M.empty
+    _ + _ = error "(Permutation a).+: not applicable"
+    negate _ = error "(Permutation a).negate: not applicable"
+    abs _ = error "(Permutation a).abs: not applicable"
+    signum _ = error "(Permutation a).signum: not applicable"
 
-inverse (P g) = P $ M.fromList $ map (\(x,y)->(y,x)) $ M.toList g
+-- |The HasInverses instance is what enables us to write @g^-1@ for the inverse of a group element.
+instance (Ord a, Show a) => HasInverses (Permutation a) where
+    inverse (P g) = P $ M.fromList $ map (\(x,y)->(y,x)) $ M.toList g
 
--- |A trick: g^-1 returns the inverse of g
-(^-) :: (Ord k, Show k) => Permutation k -> Int -> Permutation k
-g ^- n = inverse g ^ n
 
-instance (Ord a, Show a) => Fractional (Permutation a) where
-    recip = inverse
-
--- |g ~^ h returns the conjugate of g by h.
+-- |g ~^ h returns the conjugate of g by h, that is, h^-1*g*h.
 -- The tilde is meant to a mnemonic, because conjugacy is an equivalence relation.
-(~^) :: (Ord t, Show t) => Permutation t -> Permutation t -> Permutation t
+(~^) :: (Ord a, Show a) => Permutation a -> Permutation a -> Permutation a
 g ~^ h = h^-1 * g * h
 
 -- commutator
@@ -122,7 +138,7 @@ orbitP gs x = orbit (.^) x gs
 orbitV gs x = orbit (.^) x gs
 
 -- |b -^^ gs returns the orbit of the block or edge b under the action of the gs
-(-^^) :: (Ord t) => [t] -> [Permutation t] -> [[t]]
+(-^^) :: (Ord a) => [a] -> [Permutation a] -> [[a]]
 b -^^ gs = orbit (-^) b gs
 
 orbitB gs b = orbit (-^) b gs
@@ -174,9 +190,9 @@ _A n | n > 3 = [s,t]
           t = p [[1,2,3]]
 
 
--- Direct product of groups
--- Given generators for H and K, acting on sets X and Y respectively,
--- return generators for H*K, acting on the disjoint union X+Y (== Either X Y)
+-- |Given generators for groups H and K, acting on sets A and B respectively,
+-- return generators for the direct product H*K, acting on the disjoint union A+B (= Either A B)
+dp :: (Ord a, Ord b) => [Permutation a] -> [Permutation b] -> [Permutation (Either a b)]
 dp hs ks =
     [P $ M.fromList $ map (\(x,x') -> (Left x,Left x')) $ M.toList h' | P h' <- hs] ++
     [P $ M.fromList $ map (\(y,y') -> (Right y,Right y')) $ M.toList k' | P k' <- ks]
@@ -300,9 +316,9 @@ h ~^^ gs = conjClass gs h
 conjClass gs h = closure [h] [ (~^ g) | g <- gs]
 -- conjClass gs h = h ~^^ gs
 
--- |conjClassReps gs returns a conjugacy class representatives and sizes for the group generated by gs.
+-- |conjClassReps gs returns conjugacy class representatives and sizes for the group generated by gs.
 -- This implementation is only suitable for use with small groups (|G| < 10000).
-conjClassReps :: (Ord t, Show t) => [Permutation t] -> [(Permutation t, Int)]
+conjClassReps :: (Ord a, Show a) => [Permutation a] -> [(Permutation a, Int)]
 conjClassReps gs = conjClassReps' (elts gs) where
     conjClassReps' (h:hs) =
         let cc = conjClass gs h in (h, length cc) : conjClassReps' (hs \\ cc)
@@ -440,4 +456,9 @@ subgpAction gs hs =
 -- in cube gp, the subgps all appear to correspond to stabilisers of subsets, or of blocks
 
 
+-- right regular permutation representation
+rrpr gs h = rrpr' (elts gs) h
 
+rrpr' gs h = fromPairs [(g, g*h) | g <- gs]
+
+permutationMatrix xs g = [ [if x .^ g == y then 1 else 0 | y <- xs] | x <- xs ]
