@@ -43,7 +43,7 @@ class Bialgebra k b => HopfAlgebra k b where
     antipode :: Vect k b -> Vect k b
 
 
-instance (Num k, Eq b, Ord b, Show b, Algebra k b) => Num (Vect k b) where
+instance (Eq k, Num k, Eq b, Ord b, Show b, Algebra k b) => Num (Vect k b) where
     x+y = x <+> y
     negate x = neg x
     -- negate (V ts) = V $ map (\(b,x) -> (b, negate x)) ts
@@ -66,7 +66,7 @@ instance (Num k, SetCoalgebra b) => Coalgebra k b where
 -}
 
 
-instance Num k => Algebra k () where
+instance (Eq k, Num k) => Algebra k () where
     unit = wrap
     -- unit 0 = zero -- V []
     -- unit x = V [( (),x)]
@@ -74,7 +74,7 @@ instance Num k => Algebra k () where
     -- mult (V [( ((),()), x)]) = V [( (),x)]
     -- mult (V []) = zerov
 
-instance Num k => Coalgebra k () where
+instance (Eq k, Num k) => Coalgebra k () where
     counit = unwrap
     -- counit (V []) = 0
     -- counit (V [( (),x)]) = x
@@ -82,10 +82,10 @@ instance Num k => Coalgebra k () where
     -- comult (V [( (),x)]) = V [( ((),()), x)]
     -- comult (V []) = zerov
 
-unit' :: (Num k, Algebra k b) => Trivial k -> Vect k b
+unit' :: (Eq k, Num k, Algebra k b) => Trivial k -> Vect k b
 unit' = unit . unwrap -- where unwrap = counit :: Num k => Trivial k -> k
 
-counit' :: (Num k, Coalgebra k b) => Vect k b -> Trivial k
+counit' :: (Eq k, Num k, Coalgebra k b) => Vect k b -> Trivial k
 counit' = wrap . counit -- where wrap = unit :: Num k => k -> Trivial k
 
 -- unit' and counit' enable us to form tensors of these functions
@@ -94,7 +94,7 @@ counit' = wrap . counit -- where wrap = unit :: Num k => k -> Trivial k
 -- Kassel p4
 -- |The direct sum of k-algebras can itself be given the structure of a k-algebra.
 -- This is the product object in the category of k-algebras.
-instance (Num k, Ord a, Ord b, Algebra k a, Algebra k b) => Algebra k (DSum a b) where
+instance (Eq k, Num k, Ord a, Ord b, Algebra k a, Algebra k b) => Algebra k (DSum a b) where
     unit k = i1 (unit k) <+> i2 (unit k)
     -- unit == (i1 . unit) <<+>> (i2 . unit)
     mult = linear mult'
@@ -108,7 +108,7 @@ instance (Num k, Ord a, Ord b, Algebra k a, Algebra k b) => Algebra k (DSum a b)
 
 -- |The direct sum of k-coalgebras can itself be given the structure of a k-coalgebra.
 -- This is the coproduct object in the category of k-coalgebras.
-instance (Num k, Ord a, Ord b, Coalgebra k a, Coalgebra k b) => Coalgebra k (DSum a b) where
+instance (Eq k, Num k, Ord a, Ord b, Coalgebra k a, Coalgebra k b) => Coalgebra k (DSum a b) where
     counit = unwrap . linear counit'
         where counit' (Left a) = (wrap . counit) (return a)
               counit' (Right b) = (wrap . counit) (return b)
@@ -123,7 +123,7 @@ instance (Num k, Ord a, Ord b, Coalgebra k a, Coalgebra k b) => Coalgebra k (DSu
 
 -- Kassel p32
 -- |The tensor product of k-algebras can itself be given the structure of a k-algebra
-instance (Num k, Ord a, Ord b, Algebra k a, Algebra k b) => Algebra k (Tensor a b) where
+instance (Eq k, Num k, Ord a, Ord b, Algebra k a, Algebra k b) => Algebra k (Tensor a b) where
     -- unit 0 = V []
     unit x = x *> (unit 1 `te` unit 1)
     mult = linear m where
@@ -131,7 +131,7 @@ instance (Num k, Ord a, Ord b, Algebra k a, Algebra k b) => Algebra k (Tensor a
 
 -- Kassel p42
 -- |The tensor product of k-coalgebras can itself be given the structure of a k-coalgebra
-instance (Num k, Ord a, Ord b, Coalgebra k a, Coalgebra k b) => Coalgebra k (Tensor a b) where
+instance (Eq k, Num k, Ord a, Ord b, Coalgebra k a, Coalgebra k b) => Coalgebra k (Tensor a b) where
     counit = counit . (counit' `tf` counit')
     -- counit = counit . linear (\(T x y) -> counit' (return x) * counit' (return y))
     comult = assocL . (id `tf` assocR) . (id `tf` (twist `tf` id))
@@ -139,20 +139,20 @@ instance (Num k, Ord a, Ord b, Coalgebra k a, Coalgebra k b) => Coalgebra k (Ten
 
 
 -- The set coalgebra - can be defined on any set
-instance Num k => Coalgebra k EBasis where
+instance (Eq k, Num k) => Coalgebra k EBasis where
     counit (V ts) = sum [x | (ei,x) <- ts]  -- trace
     comult = fmap ( \ei -> (ei,ei) )        -- diagonal
 
 newtype SetCoalgebra b = SC b deriving (Eq,Ord,Show)
 
-instance Num k => Coalgebra k (SetCoalgebra b) where
+instance (Eq k, Num k) => Coalgebra k (SetCoalgebra b) where
     counit (V ts) = sum [x | (m,x) <- ts]  -- trace
     comult = fmap ( \m -> (m,m) )          -- diagonal
 
 
 newtype MonoidCoalgebra m = MC m deriving (Eq,Ord,Show)
 
-instance (Num k, Ord m, Mon m) => Coalgebra k (MonoidCoalgebra m) where
+instance (Eq k, Num k, Ord m, Mon m) => Coalgebra k (MonoidCoalgebra m) where
     counit (V ts) = sum [if m == MC munit then x else 0 | (m,x) <- ts]
     comult = linear cm
         where cm m = if m == MC munit then return (m,m) else return (m, MC munit) <+> return (MC munit, m)
@@ -184,13 +184,13 @@ instance Coalgebra k c => Comodule k c c where
 
 -- Kassel p57-8
 
-instance (Num k, Ord a, Ord u, Ord v, Algebra k a, Module k a u, Module k a v)
+instance (Eq k, Num k, Ord a, Ord u, Ord v, Algebra k a, Module k a u, Module k a v)
          => Module k (Tensor a a) (Tensor u v) where
     -- action x = nf $ x >>= action'
     action = linear action'
         where action' ((a,a'), (u,v)) = (action $ return (a,u)) `te` (action $ return (a',v))
 
-instance (Num k, Ord a, Ord u, Ord v, Bialgebra k a, Module k a u, Module k a v)
+instance (Eq k, Num k, Ord a, Ord u, Ord v, Bialgebra k a, Module k a u, Module k a v)
          => Module k a (Tensor u v) where
     -- action x = nf $ x >>= action'
     action = linear action'
@@ -200,7 +200,7 @@ instance (Num k, Ord a, Ord u, Ord v, Bialgebra k a, Module k a u, Module k a v)
 -- On the other hand, if a == Tensor u v, then we have overlapping instance with the earlier instance
 
 -- Kassel p63
-instance (Num k, Ord a, Ord m, Ord n, Bialgebra k a, Comodule k a m, Comodule k a n)
+instance (Eq k, Num k, Ord a, Ord m, Ord n, Bialgebra k a, Comodule k a m, Comodule k a n)
          => Comodule k a (Tensor m n) where
     coaction = (mult `tf` id) . twistm . (coaction `tf` coaction)
         where twistm x = nf $ fmap ( \((h,m), (h',n)) -> ((h,h'), (m,n)) ) x
