@@ -2,12 +2,14 @@
 
 module Math.Combinatorics.GraphAuts where
 
+import Data.Either (lefts)
 import qualified Data.List as L
 import qualified Data.Map as M
 import qualified Data.Set as S
 import Data.Maybe
 
 import Math.Common.ListSet
+import Math.Core.Utils (combinationsOf, pairs)
 import Math.Combinatorics.Graph
 -- import Math.Combinatorics.StronglyRegularGraph
 -- import Math.Combinatorics.Hypergraph -- can't import this, creates circular dependency
@@ -281,13 +283,27 @@ splitNumNbrs nbrs_g (t_src,t_trg) (c_src,c_trg) =
        -- else error (show (src_split, trg_split)) -- for debugging
 
 -- Now, every time we intersect two partitions, refine to an equitable partition
--- |Given a graph g, @graphAuts g@ returns a strong generating set for the automorphism group of g.
---
--- Note that the implementation is currently only valid for connected graphs
+
+-- |Given a graph g, @graphAuts g@ returns generators for the automorphism group of g.
+-- If g is connected, then the generators will be a strong generating set.
 graphAuts :: (Ord a) => Graph a -> [Permutation a]
-graphAuts g@(G vs es)
+graphAuts g = autsWithinComponents ++ isosBetweenComponents
+    where cs = map (inducedSubgraph g) (components g)
+          -- autsWithinComponents = concatMap graphAutsCon cs
+          autsWithinComponents = concatMap graphAuts4 cs
+          isosBetweenComponents = map swapFromIso $ concat [take 1 (graphIsos ci cj) | (ci,cj) <- pairs cs]
+          swapFromIso xys = fromPairs (xys ++ map swap xys)
+          swap (x,y) = (y,x)
+-- Using graphAuts4 instead of graphAutsCon as latter appears to have a bug, eg
+-- > graphAuts4 $ G [1..3] [[1,2],[2,3]]
+-- [[[1,3]]]
+-- > graphAutsCon $ G [1..3] [[1,2],[2,3]]
+-- []
+
+-- Automorphisms of a connected graph
+graphAutsCon g@(G vs es)
     | isConnected g = graphAuts' [] (toEquitable g $ valencyPartition g)
-    | otherwise = error "graphAuts: only implemented for connected graphs"
+    | otherwise = error "graphAutsCon: graph is not connected"
     where graphAuts' us p@((x:ys):pt) =
               let p' = L.sort $ filter (not . null) $ refine' (ys:pt) (dps M.! x)
               in level us p x ys []
@@ -305,8 +321,6 @@ graphAuts g@(G vs es)
           dps = M.fromList [(v, distancePartition g v) | v <- vs]
           es' = S.fromList es
           nbrs_g = M.fromList [(v, nbrs g v) | v <- vs]
--- To handle disconnected graphs, you not only need to find auts of each component,
--- you also need to find auts that swap components
 
 dfsEquitable (dps,es',nbrs_g) xys p1 p2 = dfs xys p1 p2 where
     dfs xys p1 p2
@@ -333,17 +347,24 @@ dfsEquitable (dps,es',nbrs_g) xys p1 p2 = dfs xys p1 p2 where
 -- AUTS OF INCIDENCE STRUCTURE VIA INCIDENCE GRAPH
 
 -- based on graphAuts as applied to the incidence graph, but modified to avoid point-block crossover auts
+
 -- |Given the incidence graph of an incidence structure between points and blocks
 -- (for example, a set system),
--- @incidenceAuts g@ returns a strong generating set for the automorphism group of the incidence structure.
+-- @incidenceAuts g@ returns generators for the automorphism group of the incidence structure.
 -- The generators are represented as permutations of the points.
 -- The incidence graph should be represented with the points on the left and the blocks on the right.
---
--- Note that the implementation is currently only valid for connected incidence graphs
+-- If the incidence graph is connected, then the generators will be a strong generating set.
 incidenceAuts :: (Ord p, Ord b) => Graph (Either p b) -> [Permutation p]
-incidenceAuts g@(G vs es) 
+incidenceAuts g = autsWithinComponents ++ isosBetweenComponents
+    where cs = map (inducedSubgraph g) (components g)
+          autsWithinComponents = concatMap incidenceAutsCon cs
+          isosBetweenComponents = map swapFromIso $ concat [take 1 (incidenceIsos ci cj) | (ci,cj) <- pairs cs]
+          swapFromIso xys = fromPairs (xys ++ map swap xys)
+          swap (x,y) = (y,x)
+
+incidenceAutsCon g@(G vs es) 
     | isConnected g = map points (incidenceAuts' [] [vs])
-    | otherwise = error "incidenceAuts: only implemented for connected incidence graphs"
+    | otherwise = error "incidenceAutsCon: graph is not connected"
     where points h = fromPairs [(x,y) | (Left x, Left y) <- toPairs h] -- filtering out the action on blocks
           incidenceAuts' us p@((x@(Left _):ys):pt) =
               -- let p' = L.sort $ filter (not . null) $ refine' (ys:pt) (dps M.! x)
@@ -351,8 +372,8 @@ incidenceAuts g@(G vs es)
               in level us p x ys []
               ++ incidenceAuts' (x:us) p'
           incidenceAuts' us ([]:pt) = incidenceAuts' us pt
-          incidenceAuts' _ (((Right _):_):_) = [] -- if we fix all the points, then the blocks must be fixed too 
-          -- incidenceAuts' _ [] = []
+          incidenceAuts' _ (((Right _):_):_) = [] -- if we fix all the points, then the blocks must be fixed too
+          incidenceAuts' _ [] = []
           level us p@(ph:pt) x (y@(Left _):ys) hs =
               let px = refine' (L.delete x ph : pt) (dps M.! x)
                   py = refine' (L.delete y ph : pt) (dps M.! y)
@@ -370,13 +391,28 @@ incidenceAuts g@(G vs es)
 
 -- !! not yet using equitable partitions, so could probably be more efficient
 
+
 -- graphIsos :: (Ord a, Ord b) => Graph a -> Graph b -> [[(a,b)]]
-graphIsos g1 g2 
+graphIsos g1 g2
+    | length cs1 /= length cs2 = []
+    | otherwise = graphIsos' cs1 cs2
+    where cs1 = map (inducedSubgraph g1) (components g1)
+          cs2 = map (inducedSubgraph g2) (components g2)
+          graphIsos' (ci:cis) cjs =
+              [iso ++ iso' | cj <- cjs,
+                             iso <- graphIsosCon ci cj,
+                             let cjs' = L.delete cj cjs,
+                             iso' <- graphIsos' cis cjs']
+          graphIsos' [] [] = [[]]
+
+-- isos between connected graphs
+graphIsosCon g1 g2 
     | isConnected g1 && isConnected g2
-        = concat [dfs [] (distancePartition g1 v1) (distancePartition g2 v2) | v2 <- vertices g2]
-    | otherwise = error "graphIsos: only implemented for connected graphs"
-    where v1 = head $ vertices g1
-          dfs xys p1 p2
+        = concat [dfs [] (distancePartition g1 v1) (distancePartition g2 v2)
+                 | v1 <- take 1 (vertices g1), v2 <- vertices g2]
+                 -- the take 1 handles the case where g1 is the null graph
+    | otherwise = error "graphIsosCon: either or both graphs are not connected"
+    where dfs xys p1 p2
               | map length p1 /= map length p2 = []
               | otherwise =
                   let p1' = filter (not . null) p1
@@ -397,10 +433,12 @@ graphIsos g1 g2
           es2 = S.fromList $ edges g2
 
 
+-- |Are the two graphs isomorphic?
+isGraphIso :: (Ord a, Ord b) => Graph a -> Graph b -> Bool
+isGraphIso g1 g2 = (not . null) (graphIsos g1 g2)
 -- !! If we're only interested in seeing whether or not two graphs are iso,
 -- !! then the cost of calculating distancePartitions may not be warranted
 -- !! (see Math.Combinatorics.Poset: orderIsos01 versus orderIsos)
-isGraphIso g1 g2 = (not . null) (graphIsos g1 g2)
 
 -- !! deprecate
 isIso g1 g2 = (not . null) (graphIsos g1 g2)
@@ -411,12 +449,26 @@ isIso g1 g2 = (not . null) (graphIsos g1 g2)
 -- we return only the action on the Lefts, and unLeft it
 -- incidenceIsos :: (Ord p1, Ord b1, Ord p2, Ord b2) =>
 --     Graph (Either p1 b1) -> Graph (Either p2 b2) -> [[(p1,p2)]]
+
 incidenceIsos g1 g2
+    | length cs1 /= length cs2 = []
+    | otherwise = incidenceIsos' cs1 cs2
+    where cs1 = map (inducedSubgraph g1) (filter (not . null . lefts) $ components g1)
+          cs2 = map (inducedSubgraph g2) (filter (not . null . lefts) $ components g2)
+          incidenceIsos' (ci:cis) cjs =
+              [iso ++ iso' | cj <- cjs,
+                             iso <- incidenceIsosCon ci cj,
+                             let cjs' = L.delete cj cjs,
+                             iso' <- incidenceIsos' cis cjs']
+          incidenceIsos' [] [] = [[]]
+
+incidenceIsosCon g1 g2
     | isConnected g1 && isConnected g2
-        = concat [dfs [] (distancePartition g1 v1) (distancePartition g2 v2) | v2@(Left _) <- vertices g2]
-    | otherwise = error "incidenceIsos: only implemented for connected graphs"
-    where v1@(Left _) = head $ vertices g1
-          dfs xys p1 p2
+        = concat [dfs [] (distancePartition g1 v1) (distancePartition g2 v2)
+                 | v1@(Left _) <- take 1 (vertices g1), v2@(Left _) <- vertices g2]
+                 -- g1 may have no vertices
+    | otherwise = error "incidenceIsos: one or both graphs not connected"
+    where dfs xys p1 p2
               | map length p1 /= map length p2 = []
               | otherwise =
                   let p1' = filter (not . null) p1
@@ -436,6 +488,9 @@ incidenceIsos g1 g2
           es1 = S.fromList $ edges g1
           es2 = S.fromList $ edges g2
 
+-- |Are the two incidence structures represented by these incidence graphs isomorphic?
+isIncidenceIso :: (Ord p1, Ord b1, Ord p2, Ord b2) =>
+     Graph (Either p1 b1) -> Graph (Either p2 b2) -> Bool
 isIncidenceIso g1 g2 = (not . null) (incidenceIsos g1 g2)
 
 {-
