@@ -157,7 +157,7 @@ lexvar :: v -> LexPoly Q v
 lexvar v = return $ Lex $ M 1 [(v,1)]
 -- lexvar = var
 
-instance (Num k, Ord v, Show v) => Algebra k (Lex v) where
+instance (Eq k, Num k, Ord v, Show v) => Algebra k (Lex v) where
     unit x = x *> return munit
     mult xy = nf $ fmap (\(a,b) -> a `mmult` b) xy
 
@@ -191,7 +191,7 @@ glexvar :: v -> GlexPoly Q v
 glexvar v = return $ Glex $ M 1 [(v,1)]
 -- glexvar = var
 
-instance (Num k, Ord v, Show v) => Algebra k (Glex v) where
+instance (Eq k, Num k, Ord v, Show v) => Algebra k (Glex v) where
     unit x = x *> return munit
     mult xy = nf $ fmap (\(a,b) -> a `mmult` b) xy
 
@@ -227,7 +227,7 @@ grevlexvar :: v -> GrevlexPoly Q v
 grevlexvar v = return $ Grevlex $ M 1 [(v,1)]
 -- grevlexvar = var
 
-instance (Num k, Ord v, Show v) => Algebra k (Grevlex v) where
+instance (Eq k, Num k, Ord v, Show v) => Algebra k (Grevlex v) where
     unit x = x *> return munit
     mult xy = nf $ fmap (\(a,b) -> a `mmult` b) xy
 
@@ -258,7 +258,7 @@ instance (Monomial a, Monomial b) => Monomial (Elim2 a b) where
     mcoprime (Elim2 a1 b1) (Elim2 a2 b2) = mcoprime a1 a2 && mcoprime b1 b2
     mdeg (Elim2 a b) = mdeg a + mdeg b
 
-instance (Num k, Ord a, Mon a, Ord b, Mon b) => Algebra k (Elim2 a b) where
+instance (Eq k, Num k, Ord a, Mon a, Ord b, Mon b) => Algebra k (Elim2 a b) where
     unit x = x *> return munit
     mult xy = nf $ fmap (\(a,b) -> a `mmult` b) xy
 
@@ -286,7 +286,7 @@ instance (Num k, Ord a, Mon a, Ord b, Mon b) => Algebra k (Elim2 a b) where
 -- This is occasionally useful.
 
 -- |bind performs variable substitution
-bind :: (Num k, MonomialConstructor m, Ord a, Show a, Algebra k a) =>
+bind :: (Eq k, Num k, MonomialConstructor m, Ord a, Show a, Algebra k a) =>
     Vect k (m v) -> (v -> Vect k a) -> Vect k a
 v `bind` f = linear (\m -> product [f x ^ i | (x,i) <- mindices m]) v
 -- V ts `bind` f = sum [c *> product [f x ^ i | (x,i) <- mindices m] | (m, c) <- ts] 
@@ -298,7 +298,7 @@ flipbind f = linear (\m -> product [f x ^ i | (x,i) <- mindices m])
 
 -- |Evaluate a polynomial at a point.
 -- For example @eval (x^2+y^2) [(x,1),(y,2)]@ evaluates x^2+y^2 at the point (x,y)=(1,2).
-eval :: (Num k, MonomialConstructor m, Eq (m v), Show v) =>
+eval :: (Eq k, Num k, MonomialConstructor m, Eq (m v), Show v) =>
     Vect k (m v) -> [(Vect k (m v), k)] -> k
 eval f vs = unwrap $ f `bind` sub
     where sub x = case lookup (var x) vs of
@@ -307,7 +307,7 @@ eval f vs = unwrap $ f `bind` sub
 
 -- |Perform variable substitution on a polynomial.
 -- For example @subst (x*z-y^2) [(x,u^2),(y,u*v),(z,v^2)]@ performs the substitution x -> u^2, y -> u*v, z -> v^2.
-subst :: (Num k, MonomialConstructor m, Eq (m u), Show u, Ord (m v), Show (m v), Algebra k (m v)) =>
+subst :: (Eq k, Num k, MonomialConstructor m, Eq (m u), Show u, Ord (m v), Show (m v), Algebra k (m v)) =>
     Vect k (m u) -> [(Vect k (m u), Vect k (m v))] -> Vect k (m v)
 subst f vs = f `bind` sub
     where sub x = case lookup (var x) vs of
@@ -392,7 +392,7 @@ infixl 7 %%
 -- In the case where the gs are a Groebner basis for an ideal I,
 -- then @f %% gs@ is the equivalence class representative of f in R/I,
 -- and is zero if and only if f is in I.
-(%%) :: (Fractional k, Monomial m, Ord m, Algebra k m) =>
+(%%) :: (Eq k, Fractional k, Monomial m, Ord m, Algebra k m) =>
      Vect k m -> [Vect k m] -> Vect k m
 f %% gs = rewrite f gs
 -- f %% gs = r where (_,r) = quotRemMP f gs
@@ -402,7 +402,7 @@ f %% gs = rewrite f gs
 -- The instance is well-defined only for scalars, and gives an error if used on other values.
 -- The purpose of this is to allow entry of fractional scalars, in expressions such as @x/2@.
 -- On the other hand, an expression such as @2/x@ will return an error.
-instance (Fractional k, Monomial m, Ord m, Algebra k m) => Fractional (Vect k m) where
+instance (Eq k, Fractional k, Monomial m, Ord m, Algebra k m) => Fractional (Vect k m) where
     recip (V [(m,c)]) | m == munit = V [(m,1/c)]
                       | otherwise = error "Polynomial recip: only defined for scalars"
     fromRational x = V [(munit, fromRational x)]
