@@ -11,7 +11,13 @@ import Math.Test.TRootSystem
 
 import Math.Test.TCore.TField
 
+import Math.Test.TAlgebras.TGroupAlgebra
+import Math.Test.TAlgebras.TOctonions
+import Math.Test.TAlgebras.TTensorAlgebra
+import Math.Test.TAlgebras.TTensorProduct
 import Math.Test.TCombinatorics.TDigraph
+import Math.Test.TCombinatorics.TFiniteGeometry
+import Math.Test.TCombinatorics.TGraphAuts
 import Math.Test.TCombinatorics.TIncidenceAlgebra
 import Math.Test.TCombinatorics.TMatroid
 import Math.Test.TCombinatorics.TPoset
@@ -21,7 +27,6 @@ import Math.Test.TNumberTheory.TPrimeFactor
 import Math.Test.TProjects.TMiniquaternionGeometry
 
 
-
 import Test.QuickCheck
 import Test.HUnit
 
@@ -37,14 +42,24 @@ testall = and
 
 quickCheckAll =
     do
-    quickCheck prop_NonCommRingNPoly
+    -- quickCheck prop_NonCommRingNPoly
     quickCheck prop_GroupPerm
     quickCheckField
+    quickCheckTensorProduct
+    quickCheckGroupAlgebra
+    quickCheckTensorAlgebra
+    putStrLn "Testing Octonions..."
+    quickCheck prop_AlgebraNonAssociative_Octonions
+    quickCheck prop_InverseLoop_Octonions
+    putStrLn "Testing miniquaternion geometries..."
     quickCheck prop_NearFieldF9
     quickCheck prop_NearFieldJ9
 
 hunitAll = runTestTT $ TestList [
+    testlistGroupAlgebra,
     testlistDigraph,
+    testlistFiniteGeometry,
+    testlistGraphAuts,
     testlistIncidenceAlgebra,
     testlistMatroid,
     testlistPoset,
